@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
+#include <math.h>
 #include <regex.h>
 #include <pthread.h>
 #include <assert.h>
@@ -29,10 +30,14 @@
 #include "server.h"
 #include "router.h"
 #include "aggregator.h"
+#include "fnv1a.h"
 
 static pthread_t aggregatorid;
 static aggregator *aggregators = NULL;
 static aggregator *lastaggr = NULL;
+static size_t prevreceived = 0;
+static size_t prevsent = 0;
+static size_t prevdropped = 0;
 static char keep_running = 1;
 
 
@@ -50,10 +55,9 @@ aggregator_new(
 
 	if (ret == NULL)
 		return ret;
-	if (interval == 0 || expire == 0) {
-		free(ret);
-		return NULL;
-	}
+
+	assert(interval != 0);
+	assert(interval < expire);
 
 	if (aggregators == NULL) {
 		aggregators = lastaggr = ret;
@@ -64,7 +68,7 @@ aggregator_new(
 	ret->interval = interval;
 	ret->expire = expire;
 	ret->tswhen = tswhen;
-	ret->bucketcnt = (expire / interval) * 2 + 1 ;
+	ret->bucketcnt = (expire + (interval - 1)) / interval + 1 + 1;
 	ret->received = 0;
 	ret->sent = 0;
 	ret->dropped = 0;
@@ -88,6 +92,8 @@ aggregator_add_compute(
 {
 	struct _aggr_computes *ac = s->computes;
 	enum _aggr_compute_type act;
+	char store = 0;
+	int pctl = 0;
 
 	if (strcmp(type, "sum") == 0) {
 		act = SUM;
@@ -99,6 +105,24 @@ aggregator_add_compute(
 		act = MIN;
 	} else if (strcmp(type, "average") == 0 || strcmp(type, "avg") == 0) {
 		act = AVG;
+	} else if (strcmp(type, "median") == 0) {
+		act = MEDN;
+		pctl = 50;
+		store = 1;
+	} else if (strncmp(type, "percentile", strlen("percentile")) == 0) {
+		pctl = atoi(type + strlen("percentile"));
+		if (pctl > 100 || pctl <= 0) {
+			return -1;
+		} else {
+			act = PCTL;
+			store = 1;
+		}
+	} else if (strcmp(type, "variance") == 0) {
+		act = VAR;
+		store = 1;
+	} else if (strcmp(type, "stddev") == 0) {
+		act = SDEV;
+		store = 1;
 	} else {
 		return -1;
 	}
@@ -112,13 +136,30 @@ aggregator_add_compute(
 	}
 
 	ac->type = act;
+	ac->percentile = (unsigned char)pctl;
 	ac->metric = strdup(metric);
 	memset(ac->invocations_ht, 0, sizeof(ac->invocations_ht));
+	ac->entries_needed = store;
 	ac->next = NULL;
 
 	return 0;
 }
 
+void
+aggregator_set_stub(
+		aggregator *s,
+		const char *stubname)
+{
+	struct _aggr_computes *ac;
+	char newmetric[METRIC_BUFSIZ];
+
+	for (ac = s->computes; ac != NULL; ac = ac->next) {
+		snprintf(newmetric, sizeof(newmetric), "%s%s", stubname, ac->metric);
+		free((void *)ac->metric);
+		ac->metric = strdup(newmetric);
+	}
+}
+
 /**
  * Adds a new metric to aggregator s.  The value from the metric is put
  * in the bucket matching the epoch contained in the metric.  In cases
@@ -138,7 +179,6 @@ aggregator_putmetric(
 	long long int epoch;
 	long long int itime;
 	int slot;
-	struct _bucket *bucket;
 	char newmetric[METRIC_BUFSIZ];
 	char *newfirstspace = NULL;
 	size_t len;
@@ -148,6 +188,8 @@ aggregator_putmetric(
 	unsigned int omhtbucket;
 	struct _aggr_computes *compute;
 	struct _aggr_invocations *invocation;
+	struct _aggr_bucket *bucket;
+	struct _aggr_bucket_entries *entries;
 
 	/* get value */
 	if ((v = strchr(firstspace + 1, ' ')) == NULL) {
@@ -180,9 +222,9 @@ aggregator_putmetric(
 			ometric = newmetric;
 		}
 
-		omhash = 2166136261UL;  /* FNV1a */
+		omhash = FNV1A_32_OFFSET;
 		for (omp = ometric; *omp != '\0'; omp++)
-			omhash = (omhash ^ (unsigned int)*omp) * 16777619;
+			omhash = (omhash ^ (unsigned int)*omp) * FNV1A_32_PRIME;
 
 		omhtbucket =
 			((omhash >> AGGR_HT_POW_SIZE) ^ omhash) &
@@ -215,14 +257,17 @@ aggregator_putmetric(
 			 * aggregator is spammed with metrics, e.g. right after
 			 * startup when other relays flush their queues.  This
 			 * approach shouldn't affect the timing of the buckets as
-			 * requested in issue #72. */
+			 * requested in issue #72.
+			 * For consistency with other tools/old carbon-aggregator
+			 * align the buckets to interval boundaries such that it is
+			 * predictable what intervals will be taken, issue #104. */
 			time(&now);
-			now -= s->expire;
+			now = ((now - s->expire) / s->interval) * s->interval;
 			invocation->expire = s->expire + (rand() % s->interval);
 
 			/* allocate enough buckets to hold the past + future */
 			invocation->buckets =
-				malloc(sizeof(struct _bucket) * s->bucketcnt);
+				malloc(sizeof(struct _aggr_bucket) * s->bucketcnt);
 			if (invocation->buckets == NULL) {
 				logerr("aggregator: out of memory creating %s from %s",
 						ometric, metric);
@@ -261,24 +306,45 @@ aggregator_putmetric(
 
 		bucket = &invocation->buckets[slot];
 		if (bucket->cnt == 0) {
-			bucket->cnt = 1;
 			bucket->sum = val;
 			bucket->max = val;
 			bucket->min = val;
 		} else {
-			bucket->cnt++;
 			bucket->sum += val;
 			if (bucket->max < val)
 				bucket->max = val;
 			if (bucket->min > val)
 				bucket->min = val;
 		}
+		entries = &bucket->entries;
+		if (compute->entries_needed) {
+			if (bucket->cnt == entries->size) {
+#define E_I_SZ 64
+				double *new = realloc(entries->values, entries->size + E_I_SZ);
+				if (new == NULL) {
+					logerr("aggregator: out of memory creating entry bucket "
+							"(%s from %s)", ometric, metric);
+				} else {
+					entries->values = new;
+					entries->size += E_I_SZ;
+				}
+			}
+			if (bucket->cnt < entries->size)
+				entries->values[bucket->cnt] = val;
+		}
+		bucket->cnt++;
 	}
 	pthread_mutex_unlock(&s->bucketlock);
 
 	return;
 }
 
+static inline int
+cmp_entry(const void *l, const void *r)
+{
+	return *(const double *)l - *(const double *)r;
+}
+
 /**
  * Checks if the oldest bucket should be expired, if so, sends out
  * computed aggregate metrics and moves the bucket to the end of the
@@ -290,11 +356,13 @@ aggregator_expire(void *sub)
 {
 	time_t now;
 	aggregator *s;
-	struct _bucket *b;
+	struct _aggr_bucket *b;
 	struct _aggr_computes *c;
 	struct _aggr_invocations *inv;
 	struct _aggr_invocations *lastinv;
+	double *values;
 	int i;
+	unsigned char j;
 	int work;
 	server *submission = (server *)sub;
 	char metric[METRIC_BUFSIZ];
@@ -362,6 +430,43 @@ aggregator_expire(void *sub)
 												inv->metric,
 												b->sum / (double)b->cnt, ts);
 										break;
+									case MEDN:
+										/* median == 50th percentile */
+									case PCTL: {
+										/* nearest rank method */
+										size_t n =
+											(int)(((double)c->percentile/100.0 *
+														(double)b->cnt) + 0.9);
+										values = b->entries.values;
+										/* TODO: lazy approach, in case
+										 * of 1 (first/last) or 2 buckets
+										 * distance we could do a
+										 * forward run picking the max
+										 * entries and returning that
+										 * iso sorting the full array */
+										qsort(values, b->cnt,
+												sizeof(double), cmp_entry);
+										snprintf(metric, sizeof(metric),
+												"%s %f %lld\n",
+												inv->metric,
+												values[n - 1],
+												b->start + s->interval);
+									}	break;
+									case VAR:
+									case SDEV: {
+										double avg = b->sum / (double)b->cnt;
+										double ksum = 0;
+										values = b->entries.values;
+										for (i = 0; i < b->cnt; i++)
+											ksum += pow(values[i] - avg, 2);
+										ksum /= (double)b->cnt;
+										snprintf(metric, sizeof(metric),
+												"%s %f %lld\n",
+												inv->metric,
+												c->type == VAR ? ksum :
+													sqrt(ksum),
+												b->start + s->interval);
+									}	break;
 								}
 								server_send(submission, strdup(metric), 1);
 								s->sent++;
@@ -383,7 +488,6 @@ aggregator_expire(void *sub)
 						}
 
 						if (isempty) {
-							int j;
 							/* see if the remaining buckets are empty too */
 							pthread_mutex_lock(&s->bucketlock);
 							for (j = 0; j < s->bucketcnt; j++) {
@@ -396,6 +500,10 @@ aggregator_expire(void *sub)
 						}
 						if (isempty) {
 							/* free and unlink */
+							if (c->entries_needed)
+								for (j = 0; j < s->bucketcnt; j++)
+									if (inv->buckets[j].entries.values)
+										free(inv->buckets[j].entries.values);
 							free(inv->metric);
 							free(inv->buckets);
 							if (lastinv != NULL) {
@@ -498,6 +606,19 @@ aggregator_get_received(void)
 	return totreceived;
 }
 
+/**
+ * Returns an approximate number of metrics received by all aggregators
+ * since the last call to this function.
+ */
+inline size_t
+aggregator_get_received_sub()
+{
+	size_t d = aggregator_get_received();
+	size_t r = d - prevreceived;
+	prevreceived += d;
+	return r;
+}
+
 /**
  * Returns an approximate number of metrics sent by all aggregators.
  */
@@ -513,6 +634,19 @@ aggregator_get_sent(void)
 	return totsent;
 }
 
+/**
+ * Returns an approximate number of metrics sent by all aggregators
+ * since the last call to this function.
+ */
+inline size_t
+aggregator_get_sent_sub()
+{
+	size_t d = aggregator_get_sent();
+	size_t r = d - prevsent;
+	prevsent += d;
+	return r;
+}
+
 /**
  * Returns an approximate number of dropped metrics by all aggregators.
  * Metrics are dropped if they are too much in the past (past expiry
@@ -529,3 +663,16 @@ aggregator_get_dropped(void)
 
 	return totdropped;
 }
+
+/**
+ * Returns an approximate number of metrics dropped by all aggregators
+ * since the last call to this function.
+ */
+inline size_t
+aggregator_get_dropped_sub()
+{
+	size_t d = aggregator_get_dropped();
+	size_t r = d - prevdropped;
+	prevdropped += d;
+	return r;
+}
