@@ -21,6 +21,8 @@
 #include <ctype.h>
 #include <regex.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
 #include <assert.h>
@@ -260,7 +262,8 @@ determine_if_regex(route *r, char *pat, int flags)
  */
 int
 router_readconfig(cluster **clret, route **rret,
-		const char *path, size_t queuesize, size_t batchsize)
+		const char *path, size_t queuesize, size_t batchsize,
+		unsigned short iotimeout)
 {
 	FILE *cnf;
 	char *buf;
@@ -607,7 +610,7 @@ router_readconfig(cluster **clret, route **rret,
 							*proto == 'f' ? CON_FILE :
 							*proto == 'u' ? CON_UDP : CON_TCP,
 							walk == (void *)1 ? NULL : walk,
-							queuesize, batchsize);
+							queuesize, batchsize, iotimeout);
 					if (newserver == NULL) {
 						logerr("failed to add server %s:%d (%s) "
 								"to cluster %s: %s\n", ip, port, proto,
@@ -708,6 +711,20 @@ router_readconfig(cluster **clret, route **rret,
 					if (cl->type == FAILOVER)
 						server_set_failover(w->server);
 				}
+			} else if (cl->type == CARBON_CH || cl->type == FNV1A_CH) {
+				/* check that replication count is actually <= the
+				 * number of servers */
+				size_t i = 0;
+				for (w = cl->members.ch->servers; w != NULL; w = w->next)
+					i++;
+				if (i <= cl->members.ch->repl_factor) {
+					logerr("invalid cluster '%s': replication count (%zd) is "
+							"larger than the number of servers (%zd)\n",
+							name, cl->members.ch->repl_factor, i);
+					free(cl);
+					free(buf);
+					return 0;
+				}
 			}
 			cl->name = strdup(name);
 			cl->next = NULL;
