@@ -43,6 +43,7 @@ struct _server {
 	int fd;
 	queue *queue;
 	size_t bsize;
+	short iotimeout;
 	const char **batch;
 	serv_ctype ctype;
 	pthread_t tid;
@@ -80,8 +81,10 @@ server_queuereader(void *d)
 	const char **metric = self->batch;
 	struct timeval start, stop;
 	struct timeval timeout;
+	int timeoutms;
 	queue *queue;
 	char idle = 0;
+	size_t *secpos = NULL;
 
 	*metric = NULL;
 	self->metrics = 0;
@@ -116,6 +119,34 @@ server_queuereader(void *d)
 				 (!self->failover && LEN_CRITICAL(self->queue))))
 		{
 			size_t i;
+
+			if (self->secondariescnt > 0) {
+				if (secpos == NULL) {
+					secpos = malloc(sizeof(size_t) * self->secondariescnt);
+					if (secpos == NULL) {
+						logerr("server: failed to allocate memory "
+								"for secpos\n");
+						continue;
+					}
+					for (i = 0; i < self->secondariescnt; i++)
+						secpos[i] = i;
+				}
+				if (!self->failover) {
+					/* randomise the failover list such that in the
+					 * grand scheme of things we don't punish the first
+					 * working server in the list to deal with all
+					 * traffic meant for a now failing server */
+					for (i = 0; i < self->secondariescnt; i++) {
+						size_t n = rand() % (self->secondariescnt - i);
+						if (n != i) {
+							size_t t = secpos[n];
+							secpos[n] = secpos[i];
+							secpos[i] = t;
+						}
+					}
+				}
+			}
+
 			/* offload data from our queue to our secondaries
 			 * when doing so, observe the following:
 			 * - avoid nodes that are in failure mode
@@ -129,9 +160,9 @@ server_queuereader(void *d)
 			queue = NULL;
 			for (i = 0; i < self->secondariescnt; i++) {
 				/* both conditions below make sure we skip ourself */
-				if (self->secondaries[i]->failure)
+				if (self->secondaries[secpos[i]]->failure)
 					continue;
-				queue = self->secondaries[i]->queue;
+				queue = self->secondaries[secpos[i]]->queue;
 				if (!self->failover && LEN_CRITICAL(queue)) {
 					queue = NULL;
 					continue;
@@ -251,8 +282,9 @@ server_queuereader(void *d)
 					fd_set fds;
 					FD_ZERO(&fds);
 					FD_SET(self->fd, &fds);
-					timeout.tv_sec = 0;
-					timeout.tv_usec = (600 + (rand() % 100)) * 1000;
+					timeoutms = self->iotimeout + (rand() % 100);
+					timeout.tv_sec = timeoutms / 1000;
+					timeout.tv_usec = (timeoutms % 1000) * 1000;
 					ret = select(self->fd + 1, NULL, &fds, NULL, &timeout);
 					if (ret == 0) {
 						/* time limit expired */
@@ -317,8 +349,9 @@ server_queuereader(void *d)
 			}
 
 			/* ensure we will break out of connections being stuck */
-			timeout.tv_sec = 0;
-			timeout.tv_usec = (600 + (rand() % 100)) * 1000;
+			timeoutms = self->iotimeout + (rand() % 100);
+			timeout.tv_sec = timeoutms / 1000;
+			timeout.tv_usec = (timeoutms % 1000) * 1000;
 			setsockopt(self->fd, SOL_SOCKET, SO_SNDTIMEO,
 					&timeout, sizeof(timeout));
 #ifdef SO_NOSIGPIPE
@@ -408,7 +441,8 @@ server_new(
 		serv_ctype ctype,
 		struct addrinfo *saddr,
 		size_t qsize,
-		size_t bsize)
+		size_t bsize,
+		unsigned short iotimeout)
 {
 	server *ret;
 
@@ -423,6 +457,7 @@ server_new(
 	ret->port = port;
 	ret->instance = NULL;
 	ret->bsize = bsize;
+	ret->iotimeout = iotimeout < 250 ? 600 : iotimeout;
 	if ((ret->batch = malloc(sizeof(char *) * (bsize + 1))) == NULL) {
 		free(ret);
 		return NULL;
